@@ -43,9 +43,9 @@ rbn.default = function(x, n = 1, ...) {
 random.graph = function(nodes, num = 1, method = "ordered", ..., debug = FALSE) {
 
   # check the generation method.
-  if (!(method %in% graph.generation.algorithms))
-    stop(paste("valid generation methods are:",
-           paste(graph.generation.algorithms, collapse = " ")))
+  if (method %!in% graph.generation.algorithms)
+    stop("valid generation methods are:",
+      paste0(" '", graph.generation.algorithms, "'"), ".")
   # check the node labels.
   check.nodes(nodes)
   # check the number of graph to generate.
@@ -82,9 +82,9 @@ cpquery = function(fitted, event, evidence, cluster = NULL, method = "ls", ...,
   if (missing(evidence))
     stop("the expression describing the evidence is missing.")
   # check the generation method.
-  if (!(method %in% cpq.algorithms))
-    stop(paste(c("valid conditional probability query methods are:\n",
-      sprintf("    %-10s %s\n", cpq.algorithms, cpq.labels[cpq.algorithms])), sep = ""))
+  if (method %!in% cpq.algorithms)
+    stop("valid conditional probability query methods are:\n",
+      sprintf("    %-10s %s\n", cpq.algorithms, cpq.labels[cpq.algorithms]))
   # check the cluster.
   if (!is.null(cluster)) {
 
@@ -149,9 +149,9 @@ cpdist = function(fitted, nodes, evidence, cluster = NULL, method = "ls", ...,
   # check debug.
   check.logical(debug)
   # check the generation method.
-  if (!(method %in% cpq.algorithms))
-    stop(paste(c("valid conditional probability query methods are:\n",
-      sprintf("    %-10s %s\n", cpq.algorithms, cpq.labels[cpq.algorithms])), sep = ""))
+  if (method %!in% cpq.algorithms)
+    stop("valid conditional probability query methods are:\n",
+      sprintf("    %-10s %s\n", cpq.algorithms, cpq.labels[cpq.algorithms]))
   # check the cluster.
   if (!is.null(cluster)) {
 
